#include "FAtoRRGConverter.h"

using namespace std;
using namespace automaton;
using namespace grammar;
using namespace alib;

namespace conversions
{

FAtoRRGConverter::FAtoRRGConverter( const FSM & automaton ) : m_automaton( automaton )
{
    if( ! automaton.isEpsilonFree( ) )
        throw AlibException( "FSM has epsilon transitions." );
}

FAtoRRGConverter::~FAtoRRGConverter( void )
{

}

RightRegularGrammar FAtoRRGConverter::convert( void )
{
    RightRegularGrammar grammar;
    map<State, Symbol> nonterminalMap;

    for( const auto & symbol : m_automaton.getInputAlphabet( ) )
        grammar.addTerminalSymbol( symbol );

    for( const auto & state : m_automaton.getStates( ) )
    {
        Symbol nt = grammar.createUniqueNonTerminalSymbol( state.getName( ) );
        nonterminalMap.insert( pair<State,Symbol>( state, nt ) );
    }

    // step 2 - create set of P in G
    for( const auto & transition : m_automaton.getTransitions( ) )
    {
        // 2a
        list<Symbol> leftSide, rightSide;
        leftSide.push_back( nonterminalMap.find( transition.getFrom( ) )->second );
        rightSide.push_back( transition.getInput( ) );
        rightSide.push_back( nonterminalMap.find( transition.getTo( ) )->second );
        grammar.addRule( Rule( leftSide, rightSide ) );

        // 2b
        if( isInSet( transition.getTo( ), m_automaton.getFinalStates( ) ) )
        {
            list<Symbol> leftSide, rightSide;
            leftSide.push_back( nonterminalMap.find( transition.getFrom( ) )->second );
            rightSide.push_back( transition.getInput( ) );

            /*
             *  Rule might already in - consider case:
             *  \delta(A,a) = { F, G }, where both F and G are final states
             */
            if( ! isInSet( Rule( leftSide, rightSide), grammar.getRules( ) ) )
                grammar.addRule( Rule( leftSide, rightSide ) );
        }
    }

    // step 3 - set start symbol of G
    grammar.setStartSymbol( nonterminalMap.find( * m_automaton.getInitialStates( ).begin( ) )->second );

    // step 4
    if( isInSet( * m_automaton.getInitialStates( ).begin( ), m_automaton.getFinalStates( ) ) )
    {
        // 4a
        if( ! grammar.isNonTerminalOnRightSideOfAnyRule( grammar.getStartSymbol( ) ) )
        {
            list<Symbol> leftSide, rightSide;
            leftSide.push_back( grammar.getStartSymbol( ) );
            grammar.addRule( Rule( leftSide, rightSide ) );
        }
        else // 4b
        {
            Symbol newStart = grammar.createUniqueNonTerminalSymbol( grammar.getStartSymbol( ).getSymbol( ), false );

            for( const auto & rule : grammar.getRules( ) )
            {
                if( rule.getLeftSide( ).front( ) == grammar.getStartSymbol( ) )
                {
                    list<Symbol> leftSide, rightSide( rule.getRightSide( ) );
                    leftSide.push_back( newStart );
                    grammar.addRule( Rule( leftSide, rightSide ) );
                }
            }
            grammar.setStartSymbol( newStart );

            list<Symbol> leftSide, rightSide;
            leftSide.push_back( newStart );
            Rule rr( leftSide, rightSide );
            grammar.addRule( Rule( leftSide, rightSide ) );
        }
    }

    return grammar;
}

} /* namespace conversions */
