/*
 * BrzozowskiDerivationRRG.h
 *
 *  Created on: 6. 3. 2014
 *      Author: tomas
 */

#ifndef BRZOZOWSKIDERIVATIONRRG_H_
#define BRZOZOWSKIDERIVATIONRRG_H_

#include <deque>
#include <set>
#include <map>

#include <alphabet/Symbol.h>
#include <grammar/Regular/RightRegularGrammar.h>
#include <regexp/RegExp.h>

#include "../../include/macros.h"
#include "../../interface/IConversionRRG.h"
#include "../../shared/Hexavigesimal.h"

#include "RegExpOptimize.h"
#include "RegExpDerivation.h"

namespace conversions
{

/**
 * Converts reg. expression to right regular grammar using brzozowski derivation algorithm.
 * Source: Melichar 2.137
 */
class BrzozowskiDerivationRRG : public IConversionRRG
{
public:
    /**
     * @param re Source regular expression.
     */
    BrzozowskiDerivationRRG( const regexp::RegExp & re );

    ~BrzozowskiDerivationRRG( void );

    /**
     * Performs conversion.
     * @return right regular grammar equivalent to source regexp.
     */
    grammar::RightRegularGrammar convert( void );

protected:
    /*
     * input regexp
     */
    const regexp::RegExp & m_re;
};

} /* namespace conversions */

#endif /* BRZOZOWSKIDERIVATIONRRG_H_ */
